package giardini.orto;

import giardini.*;
import java.io.*;
import java.util.*;

public class OrtoFactory3
implements GiardiniFactory
{
	Map<String, Class> tabellaClassi;

	public OrtoFactory3() throws InstantiationException
	{
		super();
		tabellaClassi = new HashMap<String, Class>();
	}

	public PiantaDaBordura getPiantaDaBordura(String vincoli)
	{
		try
		{
			Class clazz = tabellaClassi.get(vincoli);
			if(clazz == null)
			{
				System.err.println("Non conosco ancora la classe <" + vincoli + "> provo a caricarla");
				try
				{
					clazz = Class.forName(vincoli);
				}
				catch(ExceptionInInitializerError eiie)
				{
					eiie.printStackTrace();
					System.exit(1);
				}
				catch(LinkageError le)
				{
					le.printStackTrace();
					System.exit(1);
				}
				catch(ClassNotFoundException cnfe)
				{
					cnfe.printStackTrace();
					System.err.println("Non riesco a trovare il binario della classe <" + vincoli + ">");
					return null;
				}
				Class genitore = clazz.getSuperclass();
				//Class genitore = clazz;
				boolean eUnaPiantaDaBordura = false;
				//while( (genitore = genitore.getSuperclass()) != null)
				while(genitore != null)
				{
					String className = genitore.getName();
					System.out.println(className);
					if(className.equals("giardini.PiantaDaBordura"))
					{
						eUnaPiantaDaBordura = true;
						break;
					}
					genitore = genitore.getSuperclass();
				}
				if(!eUnaPiantaDaBordura)
				{
					System.err.println("La classe <" + vincoli + "> non e' un discendente di PiantaDaBordura: non e' compatibile con il metodo getPiantaDaBordura");
					return null;
				}
				tabellaClassi.put(vincoli, clazz);
			}

			Object o = clazz.newInstance();
			if( !(o instanceof PiantaDaBordura))
			{
				System.err.println("La classe <" + vincoli + "> non e' del tipo PiantaDaBordura");
				return null;
			}
			return (PiantaDaBordura)o;
		}
		catch(InstantiationException ie)
		{
			ie.printStackTrace();
		}
		catch(IllegalAccessException iae)
		{
			iae.printStackTrace();
		}
		return null;
	}

	static
	{
		System.out.println("Inizializzatore statico vuoto");
	}

	public static void main(String[] argv) throws IOException
	{
		OrtoFactory3 of3 = null;
		Scanner s = new Scanner(System.in);
		try
		{
			of3 = new OrtoFactory3();
		}
		catch(InstantiationException ie)
		{
			ie.printStackTrace();
			return;
		}

		String line = null;

		while( s.hasNextLine())
		{
			line = s.nextLine();
			Pianta p = of3.getPiantaDaBordura(line);
			System.out.println(p);
		}
	}

}
